@@ -21,13 +21,28 @@ from keystoneauth1 import session as ks_session
 import os_service_types
 import requestsexceptions
 from six.moves import urllib
+try:
+    import statsd
+except ImportError:
+    statsd = None
+try:
+    import prometheus_client
+except ImportError:
+    prometheus_client = None
+
 
-from openstack import _adapter
 from openstack import version as openstack_version
 from openstack import _log
 from openstack.config import _util
 from openstack.config import defaults as config_defaults
 from openstack import exceptions
+from openstack import proxy
+
+SCOPE_KEYS = {
+    'domain_id', 'domain_name',
+    'project_id', 'project_name',
+    'system_scope'
+}
 
 
 def _make_key(key, service_type):
@@ -96,7 +111,9 @@ class CloudRegion(object):
                  discovery_cache=None, extra_config=None,
                  cache_expiration_time=0, cache_expirations=None,
                  cache_path=None, cache_class='dogpile.cache.null',
-                 cache_arguments=None, password_callback=None):
+                 cache_arguments=None, password_callback=None,
+                 statsd_host=None, statsd_port=None, statsd_prefix=None,
+                 collector_registry=None):
         self._name = name
         self.region_name = region_name
         self.config = _util.normalize_keys(config)
@@ -116,6 +133,11 @@ class CloudRegion(object):
         self._cache_class = cache_class
         self._cache_arguments = cache_arguments
         self._password_callback = password_callback
+        self._statsd_host = statsd_host
+        self._statsd_port = statsd_port
+        self._statsd_prefix = statsd_prefix
+        self._statsd_client = None
+        self._collector_registry = collector_registry
 
         self._service_type_manager = os_service_types.ServiceTypes()
 
@@ -297,6 +319,7 @@ class CloudRegion(object):
         return self._get_config('service_name', service_type)
 
     def get_endpoint(self, service_type):
+        auth = self.config.get('auth', {})
         value = self._get_config('endpoint_override', service_type)
         if not value:
             value = self._get_config('endpoint', service_type)
@@ -304,7 +327,13 @@ class CloudRegion(object):
             # If endpoint is given and we're using the none auth type,
             # then the endpoint value is the endpoint_override for every
             # service.
-            value = self.config.get('auth', {}).get('endpoint')
+            value = auth.get('endpoint')
+        if (not value and service_type == 'identity'
+                and SCOPE_KEYS.isdisjoint(set(auth.keys()))):
+            # There are a small number of unscoped identity operations.
+            # Specifically, looking up a list of projects/domains/system to
+            # scope to.
+            value = auth.get('auth_url')
         return value
 
     def get_connect_retries(self, service_type):
@@ -366,6 +395,7 @@ class CloudRegion(object):
                 verify=verify,
                 cert=cert,
                 timeout=self.config.get('api_timeout'),
+                collect_timing=self.config.get('timing'),
                 discovery_cache=self._discovery_cache)
             self.insert_user_agent()
             # Using old keystoneauth with new os-client-config fails if
@@ -448,9 +478,24 @@ class CloudRegion(object):
             self.get_interface(service_type), {})
         return interface_versions.get(service_type, [])
 
+    def _get_hardcoded_endpoint(self, service_type, constructor):
+        adapter = constructor(
+            session=self.get_session(),
+            service_type=self.get_service_type(service_type),
+            service_name=self.get_service_name(service_type),
+            interface=self.get_interface(service_type),
+            region_name=self.region_name,
+        )
+        endpoint = adapter.get_endpoint()
+        if not endpoint.rstrip().rsplit('/')[-1] == 'v2.0':
+            if not endpoint.endswith('/'):
+                endpoint += '/'
+            endpoint = urllib.parse.urljoin(endpoint, 'v2.0')
+        return endpoint
+
     def get_session_client(
             self, service_type, version=None,
-            constructor=_adapter.OpenStackSDKAdapter,
+            constructor=proxy.Proxy,
             **kwargs):
         """Return a prepped keystoneauth Adapter for a given service.
 
@@ -471,34 +516,28 @@ class CloudRegion(object):
                           self.get_connect_retries(service_type))
         kwargs.setdefault('status_code_retries',
                           self.get_status_code_retries(service_type))
+        kwargs.setdefault('statsd_prefix', self.get_statsd_prefix())
+        kwargs.setdefault('statsd_client', self.get_statsd_client())
+        kwargs.setdefault('prometheus_counter', self.get_prometheus_counter())
+        kwargs.setdefault(
+            'prometheus_histogram', self.get_prometheus_histogram())
         endpoint_override = self.get_endpoint(service_type)
         version = version_request.version
         min_api_version = (
             kwargs.pop('min_version', None) or version_request.min_api_version)
         max_api_version = (
             kwargs.pop('max_version', None) or version_request.max_api_version)
+
         # Older neutron has inaccessible discovery document. Nobody noticed
         # because neutronclient hard-codes an append of v2.0. YAY!
-        if service_type == 'network':
+        # Also, older octavia has a similar issue.
+        if service_type in ('network', 'load-balancer'):
             version = None
             min_api_version = None
             max_api_version = None
             if endpoint_override is None:
-                network_adapter = constructor(
-                    session=self.get_session(),
-                    service_type=self.get_service_type(service_type),
-                    service_name=self.get_service_name(service_type),
-                    interface=self.get_interface(service_type),
-                    region_name=self.region_name,
-                )
-                network_endpoint = network_adapter.get_endpoint()
-                network_adapter.task_manager.stop()
-                if not network_endpoint.rstrip().rsplit('/')[-1] == 'v2.0':
-                    if not network_endpoint.endswith('/'):
-                        network_endpoint += '/'
-                    network_endpoint = urllib.parse.urljoin(
-                        network_endpoint, 'v2.0')
-                endpoint_override = network_endpoint
+                endpoint_override = self._get_hardcoded_endpoint(
+                    service_type, constructor)
 
         client = constructor(
             session=self.get_session(),
@@ -747,3 +786,61 @@ class CloudRegion(object):
     def get_concurrency(self, service_type=None):
         return self._get_service_config(
             'concurrency', service_type=service_type)
+
+    def get_statsd_client(self):
+        if not statsd:
+            return None
+        statsd_args = {}
+        if self._statsd_host:
+            statsd_args['host'] = self._statsd_host
+        if self._statsd_port:
+            statsd_args['port'] = self._statsd_port
+        if statsd_args:
+            return statsd.StatsClient(**statsd_args)
+        else:
+            return None
+
+    def get_statsd_prefix(self):
+        return self._statsd_prefix or 'openstack.api'
+
+    def get_prometheus_registry(self):
+        if not self._collector_registry and prometheus_client:
+            self._collector_registry = prometheus_client.REGISTRY
+        return self._collector_registry
+
+    def get_prometheus_histogram(self):
+        registry = self.get_prometheus_registry()
+        if not registry or not prometheus_client:
+            return
+        # We have to hide a reference to the histogram on the registry
+        # object, because it's collectors must be singletons for a given
+        # registry but register at creation time.
+        hist = getattr(registry, '_openstacksdk_histogram', None)
+        if not hist:
+            hist = prometheus_client.Histogram(
+                'openstack_http_response_time',
+                'Time taken for an http response to an OpenStack service',
+                labelnames=[
+                    'method', 'endpoint', 'service_type', 'status_code'
+                ],
+                registry=registry,
+            )
+            registry._openstacksdk_histogram = hist
+        return hist
+
+    def get_prometheus_counter(self):
+        registry = self.get_prometheus_registry()
+        if not registry or not prometheus_client:
+            return
+        counter = getattr(registry, '_openstacksdk_counter', None)
+        if not counter:
+            counter = prometheus_client.Counter(
+                'openstack_http_requests',
+                'Number of HTTP requests made to an OpenStack service',
+                labelnames=[
+                    'method', 'endpoint', 'service_type', 'status_code'
+                ],
+                registry=registry,
+            )
+            registry._openstacksdk_counter = counter
+        return counter
