@@ -10,11 +10,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.network.v2 import tag
 from openstack import resource
 
 
-class Port(resource.Resource, tag.TagMixin):
+class Port(resource.Resource, resource.TagMixin):
     resource_key = 'port'
     resources_key = 'ports'
     base_path = '/ports'
@@ -35,7 +34,7 @@ class Port(resource.Resource, tag.TagMixin):
         is_admin_state_up='admin_state_up',
         is_port_security_enabled='port_security_enabled',
         project_id='tenant_id',
-        **tag.TagMixin._tag_query_parameters
+        **resource.TagMixin._tag_query_parameters
     )
 
     # Properties
@@ -103,8 +102,15 @@ class Port(resource.Resource, tag.TagMixin):
     #: The ID of the project who owns the network. Only administrative
     #: users can specify a project ID other than their own.
     project_id = resource.Body('tenant_id')
+    #: Whether to propagate uplink status of the port. *Type: bool*
+    propagate_uplink_status = resource.Body('propagate_uplink_status',
+                                            type=bool)
     #: The ID of the QoS policy attached to the port.
     qos_policy_id = resource.Body('qos_policy_id')
+    #: Read-only. The port-resource-request exposes Placement resources
+    # (i.e.: minimum-bandwidth) and traits (i.e.: vnic-type, physnet)
+    # requested by a port to Nova and Placement.
+    resource_request = resource.Body('resource_request', type=dict)
     #: Revision number of the port. *Type: int*
     revision_number = resource.Body('revision_number', type=int)
     #: The IDs of any attached security groups.
