@@ -225,7 +225,54 @@ class TestNodeSetProvisionState(base.TestCase):
                                  default_microversion=None)
 
     def test_no_arguments(self):
-        self.node.set_provision_state(self.session, 'manage')
+        result = self.node.set_provision_state(self.session, 'active')
+        self.assertIs(result, self.node)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/states/provision' % self.node.id,
+            json={'target': 'active'},
+            headers=mock.ANY, microversion=None,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+    def test_manage(self):
+        result = self.node.set_provision_state(self.session, 'manage')
+        self.assertIs(result, self.node)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/states/provision' % self.node.id,
+            json={'target': 'manage'},
+            headers=mock.ANY, microversion='1.4',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+    def test_deploy_with_configdrive(self):
+        result = self.node.set_provision_state(self.session, 'active',
+                                               config_drive='abcd')
+        self.assertIs(result, self.node)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/states/provision' % self.node.id,
+            json={'target': 'active', 'configdrive': 'abcd'},
+            headers=mock.ANY, microversion=None,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+    def test_rebuild_with_configdrive(self):
+        result = self.node.set_provision_state(self.session, 'rebuild',
+                                               config_drive='abcd')
+        self.assertIs(result, self.node)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/states/provision' % self.node.id,
+            json={'target': 'rebuild', 'configdrive': 'abcd'},
+            headers=mock.ANY, microversion='1.35',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+    def test_configdrive_as_dict(self):
+        for target in ('rebuild', 'active'):
+            self.session.put.reset_mock()
+            result = self.node.set_provision_state(
+                self.session, target, config_drive={'user_data': 'abcd'})
+            self.assertIs(result, self.node)
+            self.session.put.assert_called_once_with(
+                'nodes/%s/states/provision' % self.node.id,
+                json={'target': target, 'configdrive': {'user_data': 'abcd'}},
+                headers=mock.ANY, microversion='1.56',
+                retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
 
 
 @mock.patch.object(node.Node, '_translate_response', mock.Mock())
@@ -333,6 +380,7 @@ class TestNodeVif(base.TestCase):
         super(TestNodeVif, self).setUp()
         self.session = mock.Mock(spec=adapter.Adapter)
         self.session.default_microversion = '1.28'
+        self.session.log = mock.Mock()
         self.node = node.Node(id='c29db401-b6a7-4530-af8e-20a720dee946',
                               driver=FAKE['driver'])
         self.vif_id = '714bdf6d-2386-4b5e-bd0d-bc036f04b1ef'
@@ -458,6 +506,7 @@ class TestNodeWaitForReservation(base.TestCase):
         super(TestNodeWaitForReservation, self).setUp()
         self.session = mock.Mock(spec=adapter.Adapter)
         self.session.default_microversion = '1.6'
+        self.session.log = mock.Mock()
         self.node = node.Node(**FAKE)
 
     def test_no_reservation(self, mock_fetch):
@@ -515,3 +564,122 @@ class TestNodeSetPowerState(base.TestCase):
             headers=mock.ANY,
             microversion='1.27',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+
+@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
+@mock.patch.object(node.Node, '_translate_response', mock.Mock())
+@mock.patch.object(node.Node, '_get_session', lambda self, x: x)
+class TestNodeMaintenance(base.TestCase):
+
+    def setUp(self):
+        super(TestNodeMaintenance, self).setUp()
+        self.node = node.Node.existing(**FAKE)
+        self.session = mock.Mock(spec=adapter.Adapter,
+                                 default_microversion='1.1',
+                                 retriable_status_codes=None)
+
+    def test_set(self):
+        self.node.set_maintenance(self.session)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/maintenance' % self.node.id,
+            json={'reason': None},
+            headers=mock.ANY,
+            microversion=mock.ANY)
+
+    def test_set_with_reason(self):
+        self.node.set_maintenance(self.session, 'No work on Monday')
+        self.session.put.assert_called_once_with(
+            'nodes/%s/maintenance' % self.node.id,
+            json={'reason': 'No work on Monday'},
+            headers=mock.ANY,
+            microversion=mock.ANY)
+
+    def test_unset(self):
+        self.node.unset_maintenance(self.session)
+        self.session.delete.assert_called_once_with(
+            'nodes/%s/maintenance' % self.node.id,
+            json=None,
+            headers=mock.ANY,
+            microversion=mock.ANY)
+
+    def test_set_via_update(self):
+        self.node.is_maintenance = True
+        self.node.commit(self.session)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/maintenance' % self.node.id,
+            json={'reason': None},
+            headers=mock.ANY,
+            microversion=mock.ANY)
+
+        self.assertFalse(self.session.patch.called)
+
+    def test_set_with_reason_via_update(self):
+        self.node.is_maintenance = True
+        self.node.maintenance_reason = 'No work on Monday'
+        self.node.commit(self.session)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/maintenance' % self.node.id,
+            json={'reason': 'No work on Monday'},
+            headers=mock.ANY,
+            microversion=mock.ANY)
+        self.assertFalse(self.session.patch.called)
+
+    def test_set_with_other_fields(self):
+        self.node.is_maintenance = True
+        self.node.name = 'lazy-3000'
+        self.node.commit(self.session)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/maintenance' % self.node.id,
+            json={'reason': None},
+            headers=mock.ANY,
+            microversion=mock.ANY)
+
+        self.session.patch.assert_called_once_with(
+            'nodes/%s' % self.node.id,
+            json=[{'path': '/name', 'op': 'replace', 'value': 'lazy-3000'}],
+            headers=mock.ANY,
+            microversion=mock.ANY)
+
+    def test_set_with_reason_and_other_fields(self):
+        self.node.is_maintenance = True
+        self.node.maintenance_reason = 'No work on Monday'
+        self.node.name = 'lazy-3000'
+        self.node.commit(self.session)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/maintenance' % self.node.id,
+            json={'reason': 'No work on Monday'},
+            headers=mock.ANY,
+            microversion=mock.ANY)
+
+        self.session.patch.assert_called_once_with(
+            'nodes/%s' % self.node.id,
+            json=[{'path': '/name', 'op': 'replace', 'value': 'lazy-3000'}],
+            headers=mock.ANY,
+            microversion=mock.ANY)
+
+    def test_no_reason_without_maintenance(self):
+        self.node.maintenance_reason = 'Can I?'
+        self.assertRaises(ValueError, self.node.commit, self.session)
+        self.assertFalse(self.session.put.called)
+        self.assertFalse(self.session.patch.called)
+
+    def test_set_unset_maintenance(self):
+        self.node.is_maintenance = True
+        self.node.maintenance_reason = 'No work on Monday'
+        self.node.commit(self.session)
+
+        self.session.put.assert_called_once_with(
+            'nodes/%s/maintenance' % self.node.id,
+            json={'reason': 'No work on Monday'},
+            headers=mock.ANY,
+            microversion=mock.ANY)
+
+        self.node.is_maintenance = False
+        self.node.commit(self.session)
+        self.assertIsNone(self.node.maintenance_reason)
+
+        self.session.delete.assert_called_once_with(
+            'nodes/%s/maintenance' % self.node.id,
+            json=None,
+            headers=mock.ANY,
+            microversion=mock.ANY)
