@@ -82,6 +82,8 @@ _RoleData = collections.namedtuple(
 
 class TestCase(base.TestCase):
 
+    strict_cloud = False
+
     def setUp(self, cloud_config_fixture='clouds.yaml'):
         """Run before each test method to initialize test environment."""
 
@@ -111,19 +113,10 @@ class TestCase(base.TestCase):
         vendor.write(b'{}')
         vendor.close()
 
-        test_cloud = os.environ.get('OPENSTACKSDK_OS_CLOUD', '_test_cloud_')
         self.config = occ.OpenStackConfig(
             config_files=[config.name],
             vendor_files=[vendor.name],
             secure_files=['non-existant'])
-        self.cloud_config = self.config.get_one(
-            cloud=test_cloud, validate=False)
-        self.cloud = openstack.connection.Connection(
-            config=self.cloud_config,
-            strict=False)
-        self.strict_cloud = openstack.connection.Connection(
-            config=self.cloud_config,
-            strict=True)
 
         # FIXME(notmorgan): Convert the uri_registry, discovery.json, and
         # use of keystone_v3/v2 to a proper fixtures.Fixture. For now this
@@ -443,9 +436,17 @@ class TestCase(base.TestCase):
         test_cloud = os.environ.get('OPENSTACKSDK_OS_CLOUD', cloud_name)
         self.cloud_config = self.config.get_one(
             cloud=test_cloud, validate=True, **kwargs)
-        self.conn = openstack.connection.Connection(
-            config=self.cloud_config)
-        self.cloud = self.conn
+        self.cloud = openstack.connection.Connection(
+            config=self.cloud_config, strict=self.strict_cloud)
+
+    def get_cinder_discovery_mock_dict(
+            self,
+            block_storage_version_json='block-storage-version.json',
+            block_storage_discovery_url='https://volume.example.com/'):
+        discovery_fixture = os.path.join(
+            self.fixtures_directory, block_storage_version_json)
+        return dict(method='GET', uri=block_storage_discovery_url,
+                    text=open(discovery_fixture, 'r').read())
 
     def get_glance_discovery_mock_dict(
             self,
@@ -468,6 +469,12 @@ class TestCase(base.TestCase):
             uri=compute_discovery_url,
             text=open(discovery_fixture, 'r').read())
 
+    def get_placement_discovery_mock_dict(self):
+        discovery_fixture = os.path.join(
+            self.fixtures_directory, "placement.json")
+        return dict(method='GET', uri="https://placement.example.com/",
+                    text=open(discovery_fixture, 'r').read())
+
     def get_designate_discovery_mock_dict(self):
         discovery_fixture = os.path.join(
             self.fixtures_directory, "dns.json")
@@ -480,6 +487,12 @@ class TestCase(base.TestCase):
         return dict(method='GET', uri="https://bare-metal.example.com/",
                     text=open(discovery_fixture, 'r').read())
 
+    def get_senlin_discovery_mock_dict(self):
+        discovery_fixture = os.path.join(
+            self.fixtures_directory, "clustering.json")
+        return dict(method='GET', uri="https://clustering.example.com/",
+                    text=open(discovery_fixture, 'r').read())
+
     def use_compute_discovery(
             self, compute_version_json='compute-version.json',
             compute_discovery_url='https://compute.example.com/v2.1/'):
@@ -500,6 +513,10 @@ class TestCase(base.TestCase):
             self.get_glance_discovery_mock_dict(
                 image_version_json, image_discovery_url)])
 
+    def use_placement(self):
+        self.__do_register_uris([
+            self.get_placement_discovery_mock_dict()])
+
     def use_designate(self):
         # NOTE(slaweq): This method is only meant to be used in "setUp"
         # where the ordering of the url being registered is tightly controlled
@@ -518,6 +535,15 @@ class TestCase(base.TestCase):
         self.__do_register_uris([
             self.get_ironic_discovery_mock_dict()])
 
+    def use_senlin(self):
+        # NOTE(elachance): This method is only meant to be used in "setUp"
+        # where the ordering of the url being registered is tightly controlled
+        # if the functionality of .use_senlin is meant to be used during an
+        # actual test case, use .get_senlin_discovery_mock and apply to the
+        # right location in the mock_uris when calling .register_uris
+        self.__do_register_uris([
+            self.get_senlin_discovery_mock_dict()])
+
     def register_uris(self, uri_mock_list=None):
         """Mock a list of URIs and responses via requests mock.
 
@@ -569,7 +595,7 @@ class TestCase(base.TestCase):
             key = '{method}|{uri}|{params}'.format(
                 method=method, uri=uri, params=kw_params)
             validate = to_mock.pop('validate', {})
-            valid_keys = set(['json', 'headers', 'params'])
+            valid_keys = set(['json', 'headers', 'params', 'data'])
             invalid_keys = set(validate.keys()) - valid_keys
             if invalid_keys:
                 raise TypeError(
@@ -664,7 +690,8 @@ class IronicTestCase(TestCase):
         self.uuid = str(uuid.uuid4())
         self.name = self.getUniqueString('name')
 
-    def get_mock_url(self, resource=None, append=None, qs_elements=None):
-        return super(IronicTestCase, self).get_mock_url(
-            service_type='baremetal', interface='public', resource=resource,
-            append=append, base_url_append='v1', qs_elements=qs_elements)
+    def get_mock_url(self, **kwargs):
+        kwargs.setdefault('service_type', 'baremetal')
+        kwargs.setdefault('interface', 'public')
+        kwargs.setdefault('base_url_append', 'v1')
+        return super(IronicTestCase, self).get_mock_url(**kwargs)
