@@ -11,6 +11,7 @@
 # under the License.
 
 from openstack.compute.v2 import server as server_resource
+from openstack.image.v2 import image as image_resource
 from openstack.tests.unit import base
 
 RAW_SERVER_DICT = {
@@ -66,6 +67,7 @@ RAW_SERVER_DICT = {
     'progress': 0,
     'request_ids': [],
     'security_groups': [{u'name': u'default'}],
+    'locked': True,
     'status': u'ACTIVE',
     'tenant_id': u'db92b20496ae4fbda850a689ea9d563f',
     'updated': u'2016-10-15T15:49:29Z',
@@ -93,6 +95,9 @@ RAW_GLANCE_IMAGE_DICT = {
     u'name': u'Test Monty Ubuntu',
     u'org.openstack__1__architecture': u'x64',
     u'os_type': u'linux',
+    u'os_hash_algo': u'sha512',
+    u'os_hash_value': u'fake_hash',
+    u'os_hidden': False,
     u'owner': u'610275',
     u'protected': False,
     u'schema': u'/v2/schemas/image',
@@ -372,6 +377,9 @@ class TestNormalize(base.TestCase):
                 u'com.rackspace__1__visible_rackconnect': u'1',
                 u'image_type': u'import',
                 u'org.openstack__1__architecture': u'x64',
+                u'os_hash_algo': u'sha512',
+                u'os_hash_value': u'fake_hash',
+                u'os_hidden': False,
                 u'os_type': u'linux',
                 u'schema': u'/v2/schemas/image',
                 u'user_id': u'156284',
@@ -383,6 +391,9 @@ class TestNormalize(base.TestCase):
             'min_ram': 0,
             'name': u'Test Monty Ubuntu',
             u'org.openstack__1__architecture': u'x64',
+            u'os_hash_algo': u'sha512',
+            u'os_hash_value': u'fake_hash',
+            u'os_hidden': False,
             u'os_type': u'linux',
             'owner': u'610275',
             'properties': {
@@ -397,6 +408,9 @@ class TestNormalize(base.TestCase):
                 u'com.rackspace__1__visible_rackconnect': u'1',
                 u'image_type': u'import',
                 u'org.openstack__1__architecture': u'x64',
+                u'os_hash_algo': u'sha512',
+                u'os_hash_value': u'fake_hash',
+                u'os_hidden': False,
                 u'os_type': u'linux',
                 u'schema': u'/v2/schemas/image',
                 u'user_id': u'156284',
@@ -417,6 +431,12 @@ class TestNormalize(base.TestCase):
         retval = self.cloud._normalize_image(raw_image)
         self.assertEqual(expected, retval)
 
+        # Check normalization from Image resource
+        image = image_resource.Image.existing(**RAW_GLANCE_IMAGE_DICT)
+
+        retval = self.cloud._normalize_image(image)
+        self.assertDictEqual(expected, retval)
+
     def test_normalize_servers_normal(self):
         res = server_resource.Server(
             connection=self.cloud,
@@ -424,8 +444,15 @@ class TestNormalize(base.TestCase):
         expected = {
             'OS-DCF:diskConfig': u'MANUAL',
             'OS-EXT-AZ:availability_zone': u'ca-ymq-2',
+            'OS-EXT-SRV-ATTR:host': None,
+            'OS-EXT-SRV-ATTR:hostname': None,
             'OS-EXT-SRV-ATTR:hypervisor_hostname': None,
             'OS-EXT-SRV-ATTR:instance_name': None,
+            'OS-EXT-SRV-ATTR:kernel_id': None,
+            'OS-EXT-SRV-ATTR:launch_index': None,
+            'OS-EXT-SRV-ATTR:ramdisk_id': None,
+            'OS-EXT-SRV-ATTR:reservation_id': None,
+            'OS-EXT-SRV-ATTR:root_device_name': None,
             'OS-EXT-SRV-ATTR:user_data': None,
             'OS-EXT-STS:power_state': 1,
             'OS-EXT-STS:task_state': None,
@@ -453,17 +480,23 @@ class TestNormalize(base.TestCase):
             'config_drive': u'True',
             'created': u'2015-08-01T19:52:16Z',
             'created_at': u'2015-08-01T19:52:16Z',
+            'description': None,
             'disk_config': u'MANUAL',
             'flavor': {u'id': u'bbcb7eb5-5c8d-498f-9d7e-307c575d3566'},
             'has_config_drive': True,
+            'host': None,
             'hostId': u'bd37',
             'host_id': u'bd37',
+            'host_status': None,
+            'hostname': None,
+            'hypervisor_hostname': None,
             'id': u'811c5197-dba7-4d3a-a3f6-68ca5328b9a7',
             'image': {u'id': u'69c99b45-cd53-49de-afdc-f24789eb8f83'},
             'instance_name': None,
             'interface_ip': '',
-            'hypervisor_hostname': None,
+            'kernel_id': None,
             'key_name': u'mordred',
+            'launch_index': None,
             'launched_at': u'2015-08-01T19:52:02.000000',
             'location': {
                 'cloud': '_test_cloud_',
@@ -474,6 +507,7 @@ class TestNormalize(base.TestCase):
                     'name': None},
                 'region_name': u'RegionOne',
                 'zone': u'ca-ymq-2'},
+            'locked': True,
             'metadata': {u'group': u'irc', u'groups': u'irc,enabled'},
             'name': u'mordred-irc',
             'networks': {
@@ -489,8 +523,15 @@ class TestNormalize(base.TestCase):
             'properties': {
                 'OS-DCF:diskConfig': u'MANUAL',
                 'OS-EXT-AZ:availability_zone': u'ca-ymq-2',
+                'OS-EXT-SRV-ATTR:host': None,
+                'OS-EXT-SRV-ATTR:hostname': None,
                 'OS-EXT-SRV-ATTR:hypervisor_hostname': None,
                 'OS-EXT-SRV-ATTR:instance_name': None,
+                'OS-EXT-SRV-ATTR:kernel_id': None,
+                'OS-EXT-SRV-ATTR:launch_index': None,
+                'OS-EXT-SRV-ATTR:ramdisk_id': None,
+                'OS-EXT-SRV-ATTR:reservation_id': None,
+                'OS-EXT-SRV-ATTR:root_device_name': None,
                 'OS-EXT-SRV-ATTR:user_data': None,
                 'OS-EXT-STS:power_state': 1,
                 'OS-EXT-STS:task_state': None,
@@ -498,17 +539,26 @@ class TestNormalize(base.TestCase):
                 'OS-SCH-HNT:scheduler_hints': None,
                 'OS-SRV-USG:launched_at': u'2015-08-01T19:52:02.000000',
                 'OS-SRV-USG:terminated_at': None,
-                'os-extended-volumes:volumes_attached': []},
+                'host_status': None,
+                'locked': True,
+                'os-extended-volumes:volumes_attached': [],
+                'trusted_image_certificates': None},
             'public_v4': None,
             'public_v6': None,
+            'ramdisk_id': None,
             'region': u'RegionOne',
+            'reservation_id': None,
+            'root_device_name': None,
             'scheduler_hints': None,
             'security_groups': [{u'name': u'default'}],
+            'server_groups': None,
             'status': u'ACTIVE',
+            'locked': True,
             'tags': [],
             'task_state': None,
             'tenant_id': u'db92b20496ae4fbda850a689ea9d563f',
             'terminated_at': None,
+            'trusted_image_certificates': None,
             'updated': u'2016-10-15T15:49:29Z',
             'user_data': None,
             'user_id': u'e9b21dc437d149858faee0898fb08e92',
@@ -857,6 +907,9 @@ class TestStrictNormalize(base.TestCase):
                 u'image_type': u'import',
                 u'org.openstack__1__architecture': u'x64',
                 u'os_type': u'linux',
+                u'os_hash_algo': u'sha512',
+                u'os_hash_value': u'fake_hash',
+                u'os_hidden': False,
                 u'schema': u'/v2/schemas/image',
                 u'user_id': u'156284',
                 u'vm_mode': u'hvm',
@@ -874,6 +927,7 @@ class TestStrictNormalize(base.TestCase):
     def test_normalize_servers(self):
 
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail']),
@@ -897,16 +951,21 @@ class TestStrictNormalize(base.TestCase):
             'block_device_mapping': None,
             'created': u'2015-08-01T19:52:16Z',
             'created_at': u'2015-08-01T19:52:16Z',
+            'description': None,
             'disk_config': u'MANUAL',
             'flavor': {u'id': u'bbcb7eb5-5c8d-498f-9d7e-307c575d3566'},
             'has_config_drive': True,
+            'host': None,
             'host_id': u'bd37',
+            'hostname': None,
             'hypervisor_hostname': None,
             'id': u'811c5197-dba7-4d3a-a3f6-68ca5328b9a7',
             'image': {u'id': u'69c99b45-cd53-49de-afdc-f24789eb8f83'},
             'interface_ip': u'',
             'instance_name': None,
+            'kernel_id': None,
             'key_name': u'mordred',
+            'launch_index': None,
             'launched_at': u'2015-08-01T19:52:02.000000',
             'location': {
                 'cloud': '_test_cloud_',
@@ -927,11 +986,19 @@ class TestStrictNormalize(base.TestCase):
             'power_state': 1,
             'private_v4': None,
             'progress': 0,
-            'properties': {},
+            'properties': {
+                'host_status': None,
+                'locked': True,
+                'trusted_image_certificates': None
+            },
             'public_v4': None,
             'public_v6': None,
+            'ramdisk_id': None,
+            'reservation_id': None,
+            'root_device_name': None,
             'scheduler_hints': None,
             'security_groups': [{u'name': u'default'}],
+            'server_groups': None,
             'status': u'ACTIVE',
             'tags': [],
             'task_state': None,
