@@ -11,12 +11,15 @@
 # under the License.
 
 import operator
+import six
+import tempfile
 
 from keystoneauth1 import adapter
 import mock
 import requests
 from openstack.tests.unit import base
 
+from openstack import _log
 from openstack import exceptions
 from openstack.image.v2 import image
 
@@ -36,6 +39,9 @@ EXAMPLE = {
     'status': '8',
     'tags': ['g', 'h', 'i'],
     'updated_at': '2015-03-09T12:15:57.233772',
+    'os_hash_algo': 'sha512',
+    'os_hash_value': '073b4523583784fbe01daff81eba092a262ec3',
+    'os_hidden': False,
     'virtual_size': '10',
     'visibility': '11',
     'location': '12',
@@ -47,7 +53,7 @@ EXAMPLE = {
     'url': '20',
     'metadata': {'21': '22'},
     'architecture': '23',
-    'hypervisor-type': '24',
+    'hypervisor_type': '24',
     'instance_type_rxtx_factor': 25.1,
     'instance_uuid': '26',
     'img_config_drive': '27',
@@ -83,12 +89,16 @@ EXAMPLE = {
 
 
 class FakeResponse(object):
-    def __init__(self, response, status_code=200, headers=None):
+    def __init__(self, response, status_code=200, headers=None, reason=None):
         self.body = response
         self.content = response
         self.status_code = status_code
         headers = headers if headers else {'content-type': 'application/json'}
         self.headers = requests.structures.CaseInsensitiveDict(headers)
+        if reason:
+            self.reason = reason
+        # for the sake of "list" response faking
+        self.links = []
 
     def json(self):
         return self.body
@@ -105,8 +115,10 @@ class TestImage(base.TestCase):
         self.sess.post = mock.Mock(return_value=self.resp)
         self.sess.put = mock.Mock(return_value=FakeResponse({}))
         self.sess.delete = mock.Mock(return_value=FakeResponse({}))
+        self.sess.fetch = mock.Mock(return_value=FakeResponse({}))
         self.sess.default_microversion = None
         self.sess.retriable_status_codes = None
+        self.sess.log = _log.setup_logging('openstack')
 
     def test_basic(self):
         sot = image.Image()
@@ -119,6 +131,26 @@ class TestImage(base.TestCase):
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
+        self.assertDictEqual({'created_at': 'created_at',
+                              'is_hidden': 'os_hidden',
+                              'limit': 'limit',
+                              'marker': 'marker',
+                              'member_status': 'member_status',
+                              'name': 'name',
+                              'owner': 'owner',
+                              'protected': 'protected',
+                              'size_max': 'size_max',
+                              'size_min': 'size_min',
+                              'sort': 'sort',
+                              'sort_dir': 'sort_dir',
+                              'sort_key': 'sort_key',
+                              'status': 'status',
+                              'tag': 'tag',
+                              'updated_at': 'updated_at',
+                              'visibility': 'visibility'
+                              },
+                             sot._query_mapping._mapping)
+
     def test_make_it(self):
         sot = image.Image(**EXAMPLE)
         self.assertEqual(IDENTIFIER, sot.id)
@@ -134,6 +166,9 @@ class TestImage(base.TestCase):
         self.assertEqual(EXAMPLE['status'], sot.status)
         self.assertEqual(EXAMPLE['tags'], sot.tags)
         self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
+        self.assertEqual(EXAMPLE['os_hash_algo'], sot.hash_algo)
+        self.assertEqual(EXAMPLE['os_hash_value'], sot.hash_value)
+        self.assertEqual(EXAMPLE['os_hidden'], sot.is_hidden)
         self.assertEqual(EXAMPLE['virtual_size'], sot.virtual_size)
         self.assertEqual(EXAMPLE['visibility'], sot.visibility)
         self.assertEqual(EXAMPLE['size'], sot.size)
@@ -144,7 +179,7 @@ class TestImage(base.TestCase):
         self.assertEqual(EXAMPLE['url'], sot.url)
         self.assertEqual(EXAMPLE['metadata'], sot.metadata)
         self.assertEqual(EXAMPLE['architecture'], sot.architecture)
-        self.assertEqual(EXAMPLE['hypervisor-type'], sot.hypervisor_type)
+        self.assertEqual(EXAMPLE['hypervisor_type'], sot.hypervisor_type)
         self.assertEqual(EXAMPLE['instance_type_rxtx_factor'],
                          sot.instance_type_rxtx_factor)
         self.assertEqual(EXAMPLE['instance_uuid'], sot.instance_uuid)
@@ -213,10 +248,27 @@ class TestImage(base.TestCase):
             'images/IDENTIFIER/tags/%s' % tag,
         )
 
+    def test_import_image(self):
+        sot = image.Image(**EXAMPLE)
+        json = {"method": {"name": "web-download", "uri": "such-a-good-uri"}}
+        sot.import_image(self.sess, "web-download", "such-a-good-uri")
+        self.sess.post.assert_called_with(
+            'images/IDENTIFIER/import',
+            json=json
+        )
+
+    def test_import_image_with_uri_not_web_download(self):
+        sot = image.Image(**EXAMPLE)
+        self.assertRaises(exceptions.InvalidRequest,
+                          sot.import_image,
+                          self.sess,
+                          "glance-direct",
+                          "such-a-good-uri")
+
     def test_upload(self):
         sot = image.Image(**EXAMPLE)
 
-        self.assertIsNone(sot.upload(self.sess))
+        self.assertIsNotNone(sot.upload(self.sess))
         self.sess.put.assert_called_with('images/IDENTIFIER/file',
                                          data=sot.data,
                                          headers={"Content-Type":
@@ -236,7 +288,7 @@ class TestImage(base.TestCase):
         self.sess.get.assert_called_with('images/IDENTIFIER/file',
                                          stream=False)
 
-        self.assertEqual(rv, resp.content)
+        self.assertEqual(rv, resp)
 
     def test_download_checksum_mismatch(self):
         sot = image.Image(**EXAMPLE)
@@ -266,7 +318,7 @@ class TestImage(base.TestCase):
                        stream=False),
              mock.call('images/IDENTIFIER', microversion=None)])
 
-        self.assertEqual(rv, resp1.content)
+        self.assertEqual(rv, resp1)
 
     def test_download_no_checksum_at_all2(self):
         sot = image.Image(**EXAMPLE)
@@ -292,7 +344,7 @@ class TestImage(base.TestCase):
                        stream=False),
              mock.call('images/IDENTIFIER', microversion=None)])
 
-        self.assertEqual(rv, resp1.content)
+        self.assertEqual(rv, resp1)
 
     def test_download_stream(self):
         sot = image.Image(**EXAMPLE)
@@ -308,6 +360,29 @@ class TestImage(base.TestCase):
 
         self.assertEqual(rv, resp)
 
+    def test_image_download_output_fd(self):
+        output_file = six.BytesIO()
+        sot = image.Image(**EXAMPLE)
+        response = mock.Mock()
+        response.status_code = 200
+        response.iter_content.return_value = [b'01', b'02']
+        self.sess.get = mock.Mock(return_value=response)
+        sot.download(self.sess, output=output_file)
+        output_file.seek(0)
+        self.assertEqual(b'0102', output_file.read())
+
+    def test_image_download_output_file(self):
+        sot = image.Image(**EXAMPLE)
+        response = mock.Mock()
+        response.status_code = 200
+        response.iter_content.return_value = [b'01', b'02']
+        self.sess.get = mock.Mock(return_value=response)
+
+        output_file = tempfile.NamedTemporaryFile()
+        sot.download(self.sess, output=output_file.name)
+        output_file.seek(0)
+        self.assertEqual(b'0102', output_file.read())
+
     def test_image_update(self):
         values = EXAMPLE.copy()
         del values['instance_uuid']
@@ -340,3 +415,30 @@ class TestImage(base.TestCase):
         self.assertEqual(
             sorted(value, key=operator.itemgetter('value')),
             sorted(call_kwargs['json'], key=operator.itemgetter('value')))
+
+    def test_image_find(self):
+        sot = image.Image()
+
+        self.sess._get_connection = mock.Mock(return_value=self.cloud)
+        self.sess.get.side_effect = [
+            # First fetch by name
+            FakeResponse(None, 404, headers={}, reason='dummy'),
+            # Then list with no results
+            FakeResponse({'images': []}),
+            # And finally new list of hidden images with one searched
+            FakeResponse({'images': [EXAMPLE]})
+
+        ]
+
+        result = sot.find(self.sess, EXAMPLE['name'])
+
+        self.sess.get.assert_has_calls([
+            mock.call('images/' + EXAMPLE['name'], microversion=None),
+            mock.call('/images', headers={'Accept': 'application/json'},
+                      microversion=None, params={'name': EXAMPLE['name']}),
+            mock.call('/images', headers={'Accept': 'application/json'},
+                      microversion=None, params={'os_hidden': True})
+        ])
+
+        self.assertIsInstance(result, image.Image)
+        self.assertEqual(IDENTIFIER, result.id)
