@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import testtools
+from openstack.tests.unit import base
 
 from openstack.network.v2 import auto_allocated_topology
 
@@ -20,15 +20,15 @@ EXAMPLE = {
 }
 
 
-class TestAutoAllocatedTopology(testtools.TestCase):
+class TestAutoAllocatedTopology(base.TestCase):
 
     def test_basic(self):
         topo = auto_allocated_topology.AutoAllocatedTopology
         self.assertEqual('auto_allocated_topology', topo.resource_key)
         self.assertEqual('/auto-allocated-topology', topo.base_path)
         self.assertFalse(topo.allow_create)
-        self.assertTrue(topo.allow_get)
-        self.assertFalse(topo.allow_update)
+        self.assertTrue(topo.allow_fetch)
+        self.assertFalse(topo.allow_commit)
         self.assertTrue(topo.allow_delete)
         self.assertFalse(topo.allow_list)
 
