@@ -644,6 +644,24 @@ class TestResource(base.TestCase):
         self.assertEqual(sot.alt, value2)
         self.assertEqual(sot.id, value2)
 
+    def test__alternate_id_from_other_property(self):
+        class Test(resource.Resource):
+            foo = resource.Body("foo")
+            bar = resource.Body("bar", alternate_id=True)
+
+        # NOTE(redrobot): My expectation looking at the Test class defined
+        # in this test is that because the alternate_id parameter is
+        # is being set to True on the "bar" property of the Test class,
+        # then the _alternate_id() method should return the name of that "bar"
+        # property.
+        self.assertEqual("bar", Test._alternate_id())
+        sot = Test(bar='bunnies')
+        self.assertEqual(sot.id, 'bunnies')
+        self.assertEqual(sot.bar, 'bunnies')
+        sot = Test(id='chickens', bar='bunnies')
+        self.assertEqual(sot.id, 'chickens')
+        self.assertEqual(sot.bar, 'bunnies')
+
     def test__get_id_instance(self):
         class Test(resource.Resource):
             id = resource.Body("id")
@@ -1027,6 +1045,154 @@ class TestResource(base.TestCase):
         sot._body.dirty = mock.Mock(return_value={"x": "y"})
         self.assertRaises(exceptions.MethodNotSupported, sot.commit, "")
 
+    def test_unknown_attrs_under_props_create(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+
+        sot = Test.new(**{
+            'dummy': 'value',
+        })
+        self.assertDictEqual({'dummy': 'value'}, sot.properties)
+        self.assertDictEqual(
+            {'dummy': 'value'}, sot.to_dict()['properties']
+        )
+        self.assertDictEqual(
+            {'dummy': 'value'}, sot['properties']
+        )
+        self.assertEqual('value', sot['properties']['dummy'])
+
+        sot = Test.new(**{
+            'dummy': 'value',
+            'properties': 'a,b,c'
+        })
+        self.assertDictEqual(
+            {'dummy': 'value', 'properties': 'a,b,c'},
+            sot.properties
+        )
+        self.assertDictEqual(
+            {'dummy': 'value', 'properties': 'a,b,c'},
+            sot.to_dict()['properties']
+        )
+
+        sot = Test.new(**{'properties': None})
+        self.assertIsNone(sot.properties)
+        self.assertIsNone(sot.to_dict()['properties'])
+
+    def test_unknown_attrs_not_stored(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+
+        sot = Test.new(**{
+            'dummy': 'value',
+        })
+        self.assertIsNone(sot.properties)
+
+    def test_unknown_attrs_not_stored1(self):
+        class Test(resource.Resource):
+            _store_unknown_attrs_as_properties = True
+
+        sot = Test.new(**{
+            'dummy': 'value',
+        })
+        self.assertRaises(KeyError, sot.__getitem__, 'properties')
+
+    def test_unknown_attrs_under_props_set(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+
+        sot = Test.new(**{
+            'dummy': 'value',
+        })
+
+        sot['properties'] = {'dummy': 'new_value'}
+        self.assertEqual('new_value', sot['properties']['dummy'])
+        sot.properties = {'dummy': 'new_value1'}
+        self.assertEqual('new_value1', sot['properties']['dummy'])
+
+    def test_unknown_attrs_prepare_request_unpacked(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+
+        # Unknown attribute given as root attribute
+        sot = Test.new(**{
+            'dummy': 'value',
+            'properties': 'a,b,c'
+        })
+
+        request_body = sot._prepare_request(requires_id=False).body
+        self.assertEqual('value', request_body['dummy'])
+        self.assertEqual('a,b,c', request_body['properties'])
+
+        # properties are already a dict
+        sot = Test.new(**{
+            'properties': {
+                'properties': 'a,b,c',
+                'dummy': 'value'
+            }
+        })
+
+        request_body = sot._prepare_request(requires_id=False).body
+        self.assertEqual('value', request_body['dummy'])
+        self.assertEqual('a,b,c', request_body['properties'])
+
+    def test_unknown_attrs_prepare_request_no_unpack_dict(self):
+        # if props type is not None - ensure no unpacking is done
+        class Test(resource.Resource):
+            properties = resource.Body("properties", type=dict)
+        sot = Test.new(**{
+            'properties': {
+                'properties': 'a,b,c',
+                'dummy': 'value'
+            }
+        })
+
+        request_body = sot._prepare_request(requires_id=False).body
+        self.assertDictEqual(
+            {'dummy': 'value', 'properties': 'a,b,c'},
+            request_body['properties'])
+
+    def test_unknown_attrs_prepare_request_patch_unpacked(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+            commit_jsonpatch = True
+
+        sot = Test.existing(**{
+            'dummy': 'value',
+            'properties': 'a,b,c'
+        })
+
+        sot._update(**{'properties': {'dummy': 'new_value'}})
+
+        request_body = sot._prepare_request(requires_id=False, patch=True).body
+        self.assertDictEqual(
+            {
+                u'path': u'/dummy',
+                u'value': u'new_value',
+                u'op': u'replace'
+            },
+            request_body[0])
+
+    def test_unknown_attrs_under_props_translate_response(self):
+        class Test(resource.Resource):
+            properties = resource.Body("properties")
+            _store_unknown_attrs_as_properties = True
+
+        body = {'dummy': 'value', 'properties': 'a,b,c'}
+        response = FakeResponse(body)
+
+        sot = Test()
+
+        sot._translate_response(response, has_body=True)
+
+        self.assertDictEqual(
+            {'dummy': 'value', 'properties': 'a,b,c'},
+            sot.properties
+        )
+
 
 class TestResourceActions(base.TestCase):
 
@@ -1102,7 +1268,8 @@ class TestResourceActions(base.TestCase):
                 microversion=microversion)
 
         self.assertEqual(sot.microversion, microversion)
-        sot._translate_response.assert_called_once_with(self.response)
+        sot._translate_response.assert_called_once_with(self.response,
+                                                        has_body=sot.has_body)
         self.assertEqual(result, sot)
 
     def test_put_create(self):
@@ -1336,6 +1503,60 @@ class TestResourceActions(base.TestCase):
 
         self.session.put.assert_not_called()
 
+    def test_patch_with_sdk_names(self):
+        class Test(resource.Resource):
+            allow_patch = True
+
+            id = resource.Body('id')
+            attr = resource.Body('attr')
+            nested = resource.Body('renamed')
+            other = resource.Body('other')
+
+        test_patch = [{'path': '/attr', 'op': 'replace', 'value': 'new'},
+                      {'path': '/nested/dog', 'op': 'remove'},
+                      {'path': '/nested/cat', 'op': 'add', 'value': 'meow'}]
+        expected = [{'path': '/attr', 'op': 'replace', 'value': 'new'},
+                    {'path': '/renamed/dog', 'op': 'remove'},
+                    {'path': '/renamed/cat', 'op': 'add', 'value': 'meow'}]
+        sot = Test.existing(id=1, attr=42, nested={'dog': 'bark'})
+        sot.patch(self.session, test_patch)
+        self.session.patch.assert_called_once_with(
+            '/1', json=expected, headers=mock.ANY, microversion=None)
+
+    def test_patch_with_server_names(self):
+        class Test(resource.Resource):
+            allow_patch = True
+
+            id = resource.Body('id')
+            attr = resource.Body('attr')
+            nested = resource.Body('renamed')
+            other = resource.Body('other')
+
+        test_patch = [{'path': '/attr', 'op': 'replace', 'value': 'new'},
+                      {'path': '/renamed/dog', 'op': 'remove'},
+                      {'path': '/renamed/cat', 'op': 'add', 'value': 'meow'}]
+        sot = Test.existing(id=1, attr=42, nested={'dog': 'bark'})
+        sot.patch(self.session, test_patch)
+        self.session.patch.assert_called_once_with(
+            '/1', json=test_patch, headers=mock.ANY, microversion=None)
+
+    def test_patch_with_changed_fields(self):
+        class Test(resource.Resource):
+            allow_patch = True
+
+            attr = resource.Body('attr')
+            nested = resource.Body('renamed')
+            other = resource.Body('other')
+
+        sot = Test.existing(id=1, attr=42, nested={'dog': 'bark'})
+        sot.attr = 'new'
+        sot.patch(self.session, {'path': '/renamed/dog', 'op': 'remove'})
+
+        expected = [{'path': '/attr', 'op': 'replace', 'value': 'new'},
+                    {'path': '/renamed/dog', 'op': 'remove'}]
+        self.session.patch.assert_called_once_with(
+            '/1', json=expected, headers=mock.ANY, microversion=None)
+
     def test_delete(self):
         result = self.sot.delete(self.session)
 
@@ -1534,6 +1755,53 @@ class TestResourceActions(base.TestCase):
         self.assertEqual(2, len(self.session.get.call_args_list))
         self.assertIsInstance(results[0], self.test_class)
 
+    def test_list_response_paginated_with_links_and_query(self):
+        q_limit = 1
+        ids = [1, 2]
+        mock_response = mock.Mock()
+        mock_response.status_code = 200
+        mock_response.links = {}
+        mock_response.json.side_effect = [
+            {
+                "resources": [{"id": ids[0]}],
+                "resources_links": [{
+                    "href": "https://example.com/next-url?limit=%d" % q_limit,
+                    "rel": "next",
+                }]
+            }, {
+                "resources": [{"id": ids[1]}],
+            }, {
+                "resources": [],
+            }]
+
+        self.session.get.return_value = mock_response
+
+        class Test(self.test_class):
+            _query_mapping = resource.QueryParameters("limit")
+
+        results = list(Test.list(self.session, paginated=True, limit=q_limit))
+
+        self.assertEqual(2, len(results))
+        self.assertEqual(ids[0], results[0].id)
+        self.assertEqual(ids[1], results[1].id)
+        self.assertEqual(
+            mock.call('base_path',
+                      headers={'Accept': 'application/json'}, params={
+                          'limit': q_limit,
+                      },
+                      microversion=None),
+            self.session.get.mock_calls[0])
+        self.assertEqual(
+            mock.call('https://example.com/next-url',
+                      headers={'Accept': 'application/json'}, params={
+                          'limit': [str(q_limit)],
+                      },
+                      microversion=None),
+            self.session.get.mock_calls[2])
+
+        self.assertEqual(3, len(self.session.get.call_args_list))
+        self.assertIsInstance(results[0], self.test_class)
+
     def test_list_response_paginated_with_microversions(self):
         class Test(resource.Resource):
             service = self.service_name
@@ -1901,7 +2169,10 @@ class TestResourceActions(base.TestCase):
             microversion=None)
 
         # Ensure we're done after those three items
-        self.assertRaises(StopIteration, next, results)
+        # In python3.7, PEP 479 is enabled for all code, and StopIteration
+        # raised directly from code is turned into a RuntimeError.
+        # Something about how mock is implemented triggers that here.
+        self.assertRaises((StopIteration, RuntimeError), next, results)
 
         # Ensure we only made two calls to get this done
         self.assertEqual(3, len(self.session.get.call_args_list))
@@ -2007,38 +2278,42 @@ class TestResourceActions(base.TestCase):
 
 class TestResourceFind(base.TestCase):
 
+    result = 1
+
+    class Base(resource.Resource):
+
+        @classmethod
+        def existing(cls, **kwargs):
+            response = mock.Mock()
+            response.status_code = 404
+            raise exceptions.ResourceNotFound(
+                'Not Found', response=response)
+
+        @classmethod
+        def list(cls, session, **params):
+            return None
+
+    class OneResult(Base):
+
+        @classmethod
+        def _get_one_match(cls, *args):
+            return TestResourceFind.result
+
+    class NoResults(Base):
+
+        @classmethod
+        def _get_one_match(cls, *args):
+            return None
+
+    class OneResultWithQueryParams(OneResult):
+
+        _query_mapping = resource.QueryParameters('name')
+
     def setUp(self):
         super(TestResourceFind, self).setUp()
-
-        self.result = 1
-
-        class Base(resource.Resource):
-
-            @classmethod
-            def existing(cls, **kwargs):
-                response = mock.Mock()
-                response.status_code = 404
-                raise exceptions.ResourceNotFound(
-                    'Not Found', response=response)
-
-            @classmethod
-            def list(cls, session):
-                return None
-
-        class OneResult(Base):
-
-            @classmethod
-            def _get_one_match(cls, *args):
-                return self.result
-
-        class NoResults(Base):
-
-            @classmethod
-            def _get_one_match(cls, *args):
-                return None
-
-        self.no_results = NoResults
-        self.one_result = OneResult
+        self.no_results = self.NoResults
+        self.one_result = self.OneResult
+        self.one_result_with_qparams = self.OneResultWithQueryParams
 
     def test_find_short_circuit(self):
         value = 1
@@ -2064,10 +2339,24 @@ class TestResourceFind(base.TestCase):
             self.no_results.find(
                 self.cloud.compute, "name", ignore_missing=True))
 
-    def test_find_result(self):
+    def test_find_result_name_not_in_query_parameters(self):
+        with mock.patch.object(self.one_result, 'existing',
+                               side_effect=self.OneResult.existing) \
+                as mock_existing, \
+                mock.patch.object(self.one_result, 'list',
+                                  side_effect=self.OneResult.list) \
+                as mock_list:
+            self.assertEqual(
+                self.result,
+                self.one_result.find(self.cloud.compute, "name"))
+            mock_existing.assert_called_once_with(id='name',
+                                                  connection=mock.ANY)
+            mock_list.assert_called_once_with(mock.ANY)
+
+    def test_find_result_name_in_query_parameters(self):
         self.assertEqual(
             self.result,
-            self.one_result.find(self.cloud.compute, "name"))
+            self.one_result_with_qparams.find(self.cloud.compute, "name"))
 
     def test_match_empty_results(self):
         self.assertIsNone(resource.Resource._get_one_match("name", []))
